// *********************************************************************
//							===============
//							   Stellarium
//							===============
// *********************************************************************
/*
   Copyright (C) 2001 Fabien Chreau chereau@free.fr

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/
// *********************************************************************

// I'm trying to comment the source but my english isn't perfect, so be tolerant 
// and please correct all the mistakes you find. ;)

#include <GL/glut.h>
#include <time.h>

#include "astroLib/AstroOps.h"
#include "astroLib/DateOps.h"

#include "tga.h"
#include "text.h"
#include "LesEtoiles.h"
#include "Affichage.h"
#include "Variables.h"
#include "LesConstellations.h"
#include "LesPlanetes.h"
#include "Nebuleuse.h"
#include "LesNebuleuses.h"
#include "texture.h"

// *********************************************************************
LesEtoiles VouteCeleste;			// Class to manage all the stars from HR catalog
LesConstellations ConstellCeleste;	// Class to manage the constellation boundary and name
LesPlanetes SystemePlanete;			// Class to manage the planets
LesNebuleuses messiers;
// *********************//temporaire/////////***************************
unsigned int HRtemp=0;
char HRstemp[4];
///////////////////////

// *************** Read config infos on config.txt ********************
int initConfig()
{	FILE * pFile;
	char c;
	char cmd[50];
	bool validLine=false;
	int d,m,y,hour,min,sec;
	int timeZone=0;
	// init the time parameters with current time and date
	time_t rawtime;
	tm * ptm;
	time ( &rawtime );
	ptm = gmtime ( &rawtime );
	y=ptm->tm_year+1900;
	m=ptm->tm_mon+1;
	d=ptm->tm_mday;
	hour=ptm->tm_hour;
	min=ptm->tm_min;
	sec=ptm->tm_sec;

	pFile=fopen("../config.txt","rt");
	if (pFile==NULL) printf("ERROR WHILE LOADING CONFIG.TXT\n");
	// find a valid line (remove comments)
	for(;;)
	{	while (!validLine)
		{	c=getc(pFile);
			if (c=='#')
			{	while (c!='\n')
				{	c=getc(pFile);
					//printf("%c",c);
					if (c==EOF)
					{	fclose(pFile);
						// recalc the julian date
						JDay=DateOps::dmyToDay(d,m,y);	// store it in the global variable Jday
						JDay+=((double)hour*0.0416666667+((double)min+(double)sec/60)*0.0006944444);
						return 0;
					}
				}
			}
			else
			{	if (c==EOF)
				{	fclose(pFile);
					// recalc the julian date
					JDay=DateOps::dmyToDay(d,m,y);	// store it in the global variable Jday
					JDay+=((double)hour*0.0416666667+((double)min+(double)sec/60)*0.0006944444);
					return 0;
				}
				else
				{	if (c!='\n')
					{	validLine=true;
						ungetc(c,pFile);
					}
				}
			}
		}
		fscanf(pFile,"%s ",&cmd);
		if (!strcmp(cmd,"date"))
		{	fscanf(pFile,"%d/%d/%d\n",&m,&d,&y);
		}
		if (!strcmp(cmd,"time"))
		{	fscanf(pFile,"%d:%d\n",&hour,&min,&sec);
		}
		if (!strcmp(cmd,"longitude"))
		{	fscanf(pFile,"%f\n",&Longitude);
			if (Longitude<0) Longitude=360+Longitude;
			LeLieu.setLongitude((long)Longitude);
		}
		if (!strcmp(cmd,"latitude"))
		{	fscanf(pFile,"%f\n",&Latitude);	
			Latitude=90-Latitude;
			LeLieu.setLatitude((long)Latitude);	
		}
		if (!strcmp(cmd,"timeZone"))
		{	fscanf(pFile,"%f\n",&timeZone);	
			LeLieu.setTimeZone(timeZone);
		}
		validLine=false;
	}
	return 1;
}
// *********************************************************************
void init();

// *******  Calc the Zenith position for the location and Time  ********
void CalculeZenith()
{	RaZenith = -AstroOps::greenwichSiderealTime(JDay) + LeLieu.longitude();
	RaZenith = AstroOps::normalizeRadians(RaZenith);
	DeZenith = Astro::PI_OVER_TWO - LeLieu.latitude();
	//printf("RAzenith=%f DEzenith=%f\n",RaZenith*180/PI,DeZenith*180/PI);
}

// ***************  Print a beautiful console logo !! ******************
void AfficheIntro(void)
{	printf("\n");
	printf("    -----------------------------------------\n");
	printf("    | ## ### ### #   #   ### ###  #  # # # #|\n");
	printf("    | #   #  ##  #   #   ### ##   #  # # ###|\n");
	printf("    |##   #  ### ### ### # # # #  #  ### # #|\n");
	printf("    |                     %s|\n",appName);
	printf("    -----------------------------------------\n\n");
	printf("Copyright (C) 2002 Fabien Chereau chereau@free.fr\n\n");
	printf("Please send bug report & comments to stellarium@free.fr\n\n");
};

// *********************************************************************
// Increment/decrement smoothly the vision field and position
void MajVariables(void)
{	if (deltaOuv)						// if we are zooming in or out
	{	Ouv+=deltaOuv;					// change current zoom
		if (Ouv < 0.001) Ouv-=deltaOuv;	// stop on the max and min limits
		if (Ouv > 100) Ouv-=deltaOuv;
	}
	if (deltaRa) AzVision-=deltaRa;		// if we are mooving in the Azimthal angle (left/right)
	if (deltaDe)
	{	AltVision+=deltaDe;
		if (AltVision>=PI/2) AltVision-=deltaDe;
		if (AltVision<-PI/2) AltVision-=deltaDe;
	}
	// to prevent strange behaviour, change the speed of mooving (IN REAL TIME)
	// in function of the field of view : more it is zoomed, more the mooving speed is low (in angle)
	float depl=0.6/(Fps+1);
	if (deltaRa<0)
	{	deltaRa = -depl*Ouv/30;
	}
	else 
	{	if (deltaRa>0)
		{	deltaRa = depl*Ouv/30;
		}
	}
	if (deltaDe<0) 
	{	deltaDe = -depl*Ouv/30;
	}
	else 
	{	if (deltaDe>0)
		{	deltaDe = depl*Ouv/30;
		}
	}
	if (deltaOuv<0)
	{	deltaOuv = -depl*Ouv*5;
	}
	else 
	{	if (deltaOuv>0)
		{	deltaOuv = depl*Ouv*5;
		}
	}
}

// *********************************************************************
// Moove the Camera to the aim
void GoToAim(void)
{	//not done yet....
}

// *********************************************************************
// Increment time every second for real time mode and every frame for accelered time mode
void Majtemps(void)
{	frame++;
	DeltaTemps=glutGet(GLUT_ELAPSED_TIME)-timefr;
	timefr+=DeltaTemps;
	if (Flaganim)								// real time animation
	{	JDay+=SECONDE*(float)DeltaTemps/1000.0;
	}
	if (timefr-timebase > 1000) 
	{	Fps=frame*1000.0/(timefr-timebase);		// Calc the PFS rate
		frame = 0;
		timebase+=1000;
		//JDay+=SECONDE;
	}

	if (Flaganimmin)							// accelered time animation
	{	JDay+=MINUTE*60*24;
		SystemePlanete.Calculer(JDay,LeLieu);
	}
}

// *****************  Display all the text informations  ******************
void AfficheTexte(void)
{	setOrthographicProjection(winW, winH);	// 2D coordinate
	glPushMatrix();
	glLoadIdentity();

	AfficheHoriLine();				// Draw the horizontal line

	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);

	if (FlagSelect)		// An object is selected : we print the information
	{	
		if (ObjetSelectionne.type==0)	//Star
		{	glColor3f(0.4f,0.7f,0.3f);
			sprintf(infoObjet,"Info : %s",ObjetSelectionne.CommonName);	
			glPrint(3,lineSizeOf(FONT1)+7.*winW/1024,infoObjet,FONT1);
			
			sprintf(infoObjet,"Name :%s",ObjetSelectionne.Nom);	
			glPrint(3,lineSizeOf(FONT1)*2+7.*winW/1024,infoObjet,FONT1);		
	
			sprintf(infoObjet,"HR  : %.4d",ObjetSelectionne.HR);
			glPrint(3,lineSizeOf(FONT1)*3+7.*winW/1024,infoObjet,FONT1);
			
			sprintf(infoObjet,"RA  : %.2dh %.2dm %.2fs",ObjetSelectionne.RAh,ObjetSelectionne.RAm,ObjetSelectionne.RAs);	
			glPrint(3,lineSizeOf(FONT1)*4+7.*winW/1024,infoObjet,FONT1);

			sprintf(infoObjet,"DE  : %.2fdeg",ObjetSelectionne.DE*180/PI);	
			glPrint(3,lineSizeOf(FONT1)*5+7.*winW/1024,infoObjet,FONT1);	

			sprintf(infoObjet,"Mag : %.2f",ObjetSelectionne.Mag);	
			glPrint(3,lineSizeOf(FONT1)*6+7.*winW/1024,infoObjet,FONT1);
		}
		if (ObjetSelectionne.type==1)	//Nebulae
		{	glColor3f(0.4f,0.5f,0.8f);

			sprintf(infoObjet,"Name: %s",ObjetSelectionne.Nom);	
			glPrint(3,lineSizeOf(FONT1)+7.*winW/1024,infoObjet,FONT1);		
	
			sprintf(infoObjet,"NGC : %d",ObjetSelectionne.NGCNum);
			glPrint(3,lineSizeOf(FONT1)*2+7.*winW/1024,infoObjet,FONT1);
			
			sprintf(infoObjet,"RA  : %.2dh %.2dm %.2fs",ObjetSelectionne.RAh,ObjetSelectionne.RAm,ObjetSelectionne.RAs);	
			glPrint(3,lineSizeOf(FONT1)*3+7.*winW/1024,infoObjet,FONT1);

			sprintf(infoObjet,"DE  : %.2fdeg",ObjetSelectionne.DE*180/PI);	
			glPrint(3,lineSizeOf(FONT1)*4+7.*winW/1024,infoObjet,FONT1);	

			sprintf(infoObjet,"Mag : %.2f",ObjetSelectionne.Mag);	
			glPrint(3,lineSizeOf(FONT1)*5+7.*winW/1024,infoObjet,FONT1);
		}
		if (ObjetSelectionne.type==2)	//Planet
		{	glColor3f(1.0f,0.3f,0.3f);

			sprintf(infoObjet,"Name: %s",ObjetSelectionne.Nom);	
			glPrint(3,lineSizeOf(FONT1)+7.*winW/1024,infoObjet,FONT1);		
	
			sprintf(infoObjet,"RA  : %.2dh %.2dm %.2fs",ObjetSelectionne.RAh,ObjetSelectionne.RAm,ObjetSelectionne.RAs);	
			glPrint(3,lineSizeOf(FONT1)*2+7.*winW/1024,infoObjet,FONT1);

			sprintf(infoObjet,"DE  : %.2fdeg",ObjetSelectionne.DE*180/PI);	
			glPrint(3,lineSizeOf(FONT1)*3+7.*winW/1024,infoObjet,FONT1);	

			sprintf(infoObjet,"Distance : %.4f AU",ObjetSelectionne.Distance);
			glPrint(3,lineSizeOf(FONT1)*4+7.*winW/1024,infoObjet,FONT1);
		}
	}

	if (FlagHelp) AfficheHelp();			// Draw the help screen
	if (FlagInfo) AfficheInfo();			// Draw the Information screen

	glColor3f(0.0f,0.8f,0.9f);				// Text color

	int jour,mois;							// Print the date and time
	long annee;
	DateOps::dayToDmy((long)JDay,jour,mois,annee);
	double reste=JDay-(int)DateOps::dmyToDay(jour,mois,annee);
	double heure=reste*24;
	double minute=(double)(heure-(int)heure)*60;
	double seconde=(minute-(int)minute)*60;
	char sDate[30];
	sprintf(sDate,"%.2d/%.2d/%.4d   %.2d:%.2d:%.2d",jour,mois,(int)annee,(int)heure,(int)minute,(int)seconde);
	glPrint(3,3*winW/1024,sDate,FONT1);
		
	char speed[15];
	sprintf(speed,"FPS : %4.2f",Fps);
	glPrint(winW-charSizeOf(FONT2)*strlen(speed),3*winW/1024,speed,FONT2);
	
	char sOuv[15];
	sprintf(sOuv,"fov=%.3f", Ouv);
	glPrint(winW-charSizeOf(FONT2)*(strlen(sOuv)+strlen(speed)+2),3*winW/1024,sOuv,FONT2);

	if (winW>700) glPrint((winW-charSizeOf(appName,FONT1))/2,3*winW/1024,appName,FONT1);


	// display the properties of the item of the menu which have the mouse over if it is the case
	if (FlagTexteMenu) glPrint(3*winW/1024,winH-47*winW/1024,stexteMenu,FONT1);

	glPopMatrix();
	resetPerspectiveProjection();			// Restore the other coordinate
}


/*******************  My coordinate conventions :  *******************

Always count the RA angle from z axis to x axis 

		y
		|
		|   +A
		|DE.|
		| . |
		|___|____x
       / .  | .
     / RA . |.
   /...... .|
 z
						*/

// ****************  Change the coord system to altazimutal  *********
void Passe_en_altazimutal()
{	glLoadIdentity();
	RADE_to_XYZ(AzVision, AltVision, XYZVisionAltAz);
	// Calc the equatorial coordinate of the direction of vision wich was in Altazimuthal coordinate
	XYZVision=XYZVisionAltAz;
	AltAz_to_equ(XYZVision,RaZenith,DeZenith);
	// Change it... the vertical direction is always the vector 0,1,0 in Altazimuthal coordinate
	gluLookAt(0, 0, 0, 			// Observer position
		      XYZVisionAltAz[0],XYZVisionAltAz[1],XYZVisionAltAz[2],	// direction of vision
			  0,1,0);			// Vertical vector
}

// *****************  Change the coord system to equatorial  **********
void Passe_en_equatorial()
{	glLoadIdentity();
	Passe_en_altazimutal();
	// Make the 2 rotations to transform the Altazimuthal system to Equatorial system
	glRotatef(90-DeZenith*180/PI,1,0,0);
	glRotatef(RaZenith*180/PI,0,1,0);
}


// ************************  Main display loop  ************************
void glutDisplay(void)
{	// Update the field of view
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(Ouv, (double) winW / winH, 1, 10000);
	glutPostRedisplay();
	glMatrixMode(GL_MODELVIEW);

	// Update the position of observation and time etc...
	MajVariables();
	Majtemps();
	if (FlagDeplacement) GoToAim();
	CalculeZenith();

	// Execute all the drawing function in the correct order from the more far to nearest objects

	// ---- Equatorial Coordinates
	Passe_en_equatorial();
	if (FlagVoieLactee) AfficheVoieLactee();			// Draw the milky way --> init the buffers
	messiers.Afficher();				// Draw the Messiers Objects
	//ConstellCeleste.Afficher("UMa");					// Draw one constellation only
	if (FlagConst) ConstellCeleste.Afficher();			// Draw all the constellations
	if (FlagVoute) VouteCeleste.Afficher(7.5);			// Draw the stars
	if (FlagPlanetes) SystemePlanete.Afficher();		// Draw the planets
	if (FlagLignes) AfficheMeridiens();					// Draw the meridian lines
	if (FlagLignes) AfficheParalleles();				// Draw the parallel lines
	if (FlagEquateurCeleste) AfficheEquateurCeleste();	// Draw the celestial equator line
	if (FlagEcliptique) AfficheEcliptique();			// Draw the ecliptic line
	if (FlagNomConst) ConstellCeleste.AfficherNom();	// Draw the names of the constellations
	if (FlagSelect) AffichePointeur(Pointeur,ObjetSelectionne.Size*10/Ouv,ObjetSelectionne.RGB,ObjetSelectionne.type);	// Draw the star pointer

	// ---- AltAzimutal Coordinates
	Passe_en_altazimutal();
	if (FlagLignesAzimuth) AfficheMeridiensAzimuth();	// Draw the "Altazimuthal meridian" lines
	if (FlagLignesAzimuth) AfficheParallelesAzimuth();	// Draw the "Altazimuthal parallel" lines
	if (FlagCiel) AfficheCiel();						// Draw the blue sky
	if (FlagSol) AfficheDecor(2);						// Draw the mountains
	if (FlagSol) AfficheSol();							// Draw the ground
	if (FlagHorizon) AfficheHorizon();					// Draw the fog
	if (FlagCardinaux) AfficheCardinaux();				// Daw the cardinal points

	// ---- 2D Displays
	if (FlagTexte) AfficheTexte();						// Print all the text
	if (FlagMenu) AfficheMenu();						// Draw the menu bar
	//if (FlagSelect) AfficheMenuSelect(ObjetSelectionne.type);	// Draw the Center button 
	glutSwapBuffers();
}

// ************************  On resize  *******************************

void glutResize(int w, int h)
{	if (!h) return;
	winW = w;
	winH = h;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glViewport(0, 0, winW, winH);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(Ouv, (double) winW / winH, 1, 10000);	// Update the ratio
	glutPostRedisplay();
	glMatrixMode(GL_MODELVIEW);
}

// *******************  Handle the Flag keys  **************************
void processNormalKeys(unsigned char key, int x, int y) 
{	switch (key) 			// toggle the different functions
	{	case 27 : exit(0); kill_fonts(); break;
		case 'Z' :
		case 'z' : FlagHorizon=!FlagHorizon; break;
		case 'S' :
		case 's' : FlagSol=!FlagSol; break;
		case 'V' :
		case 'v' : FlagVoieLactee=!FlagVoieLactee; break;
		case 'E' :
		case 'e' : FlagVoute=!FlagVoute; break;
		case 'L' :
		case 'l' : FlagLignes=!FlagLignes; break;
		case 'C' :
		case 'c' : FlagCiel=!FlagCiel; break;
		case 'K' :
		case 'k' : FlagConst=!FlagConst; break;
		case 'T' :
		case 't' : FlagTexte=!FlagTexte; break;
		case 'A' :
		case 'a' : FlagCardinaux=!FlagCardinaux; break;
		case 'R' :
		case 'r' : FlagEcliptique=!FlagEcliptique; break;
		case 'F' :
		case 'f' : FlagEquateurCeleste=!FlagEquateurCeleste; break;
		case 'J' :
		case 'j' : FlagLignesAzimuth=!FlagLignesAzimuth; break;
		case '+' : coefciel+=0.01;
					if (coefciel>1) coefciel=1; break;
		case '-' : coefciel-=0.01;
					if (coefciel<0) coefciel=0; break;
		case '1' : Flaganim=!Flaganim; break;
		case '0' : Flaganimmin=!Flaganimmin; break;
		case 'N' :
		case 'n' : FlagNomConst=!FlagNomConst; break;
		case 'M' :
		case 'm' : FlagMenu=!FlagMenu; 
				   FlagTexteMenu = !FlagTexteMenu; break;
		case 'H' :
		case 'h' : FlagHelp=!FlagHelp;
					if (FlagInfo) FlagInfo=false;
					if (!FlagTexte) FlagTexte=true;
					break;
		case 'I' :
		case 'i' : FlagInfo=!FlagInfo; 
					if (FlagHelp) FlagHelp=false;
					break;
		case 'P' :
		case 'p' : FlagPlanetes=!FlagPlanetes; 
					break;
		case 'D' :
		case 'd' : FlagNeb=!FlagNeb; 
					break;
	}
}

// ************************  Initialisation  ******************************
void glInit (void)
{	winW=glutGet(GLUT_SCREEN_WIDTH);
	winH=glutGet(GLUT_SCREEN_HEIGHT);
	printf("%d %d ",winW,winH);

	printf("Loading textures...\n");
	tga_t image;
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	//glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);		// init the blending function parameters (read the doc to understand it ;) ...)

	// If the pictures are in 32 bit format --> Alpha
	texIds[1] = createTexture("sol2");
	texIds[2] = createTexture("voielactee256x256");
	texIds[3] = createTexture("horizon3");
	texIds[4] = createTexture("ciel");
		
	texIds[6] = createTexture("n");
	texIds[7] = createTexture("s");
	texIds[8] = createTexture("e");
	texIds[9] = createTexture("o");
	texIds[10]= createTexture("zenith");
	texIds[11]= createTexture("nadir");
	texIds[12]= createTexture("pointeur2");
	texIds[13]= createTexture("Bouton1a2");
	texIds[14]= createTexture("Bouton2a2");
	texIds[15]= createTexture("Bouton3a2");
	texIds[16]= createTexture("Bouton4a2");
	texIds[17]= createTexture("Bouton5a2");
	texIds[18]= createTexture("Bouton6a2");
	texIds[19]= createTexture("Bouton7a2");
	texIds[20]= createTexture("Bouton8a2");
	texIds[21]= createTexture("Bouton9a2");
	texIds[22]= createTexture("Bouton10a2");
	texIds[23]= createTexture("Bouton11a2");
	texIds[24]= createTexture("Bouton12a2");

	texIds[25]= createTexture("etoile32x32");
	texIds[26]= createTexture("pointeur4");
	texIds[27]= createTexture("pointeur5");

	texIds[30]= createTexture("spacefont");

	texIds[31]= createTexture("montagne1");
	texIds[32]= createTexture("montagne2");
	texIds[33]= createTexture("montagne3");
	texIds[34]= createTexture("montagne4");

	texIds[35]= createTexture("backmenu");

	texIds[40]= createTexture("sun");
	texIds[41]= createTexture("mercure");
	texIds[42]= createTexture("venus");
	texIds[44]= createTexture("mars");
	texIds[45]= createTexture("jupiter");
	texIds[46]= createTexture("saturn");
	texIds[47]= createTexture("saturneAnneaux128x128");
	texIds[48]= createTexture("halo");
	texIds[49]= createTexture("lune");
	texIds[50]= createTexture("haloLune");

	if (messiers.LireTexture()==0) printf("Error while loading messier Texture\n");
	
	build_fonts();	// create the font texture

}
// ********************  Handle the special keys  **********************
void pressKey(int key, int x, int y) 
{	float depl=0.4/(Fps+1);			// depl is used to maintain a constant speed whatever is the FPS
	switch (key) 
	{	// Direction and zoom deplacements
		case GLUT_KEY_LEFT : deltaRa = -depl*((float)Ouv/30);break;
		case GLUT_KEY_RIGHT : deltaRa = depl*((float)Ouv/30);break;
		case GLUT_KEY_UP : deltaDe = depl*((float)Ouv/30);break;
		case GLUT_KEY_DOWN : deltaDe = -depl*((float)Ouv/30);break;
		case GLUT_KEY_PAGE_UP : deltaOuv = -depl*Ouv/20;break;
		case GLUT_KEY_PAGE_DOWN : deltaOuv = depl*Ouv/20;break;

		// Switch to "windowed" Fullscreen mode (not a real fullscreen mode, only hide the desktop)
		case GLUT_KEY_F1:  
			glutFullScreen();
			break;
		// Switch to windowed mode
		case GLUT_KEY_F2:  
			winW = 640;winH = 480;
			glutReshapeWindow(winW,winH);
			break;
			
		// Switch to real fullscreen mode with different parameters (not very stable)
		case GLUT_KEY_F3:
			glutGameModeString("640x480:16");				// define resolution, color depth
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))	// enter full screen
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:16");		// register callbacks again 
				init();										// and init OpenGL context
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F4:
			glutGameModeString("800x600:16");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:16");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F5:
			glutGameModeString("1024x768:16");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:16");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F6:
			glutGameModeString("640x480:32");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F7:
			glutGameModeString("800x600:32");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;
		case GLUT_KEY_F8:
			glutGameModeString("1024x768:32");
			if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)) 
			{	glutEnterGameMode();
				sprintf(gameModeString,"640x480:32");
				init();
			}
			else glutGameModeString(gameModeString);
			break;

		// Leave real fullscreen mode
		case GLUT_KEY_F9:  
			glutLeaveGameMode();
			break;
		}
}

// *******************  Stop mooving and zooming  **********************
void releaseKey(int key, int x, int y)
{	// When a deplacement key is released stop mooving
		switch (key)
	{	case GLUT_KEY_LEFT :
		case GLUT_KEY_RIGHT :	deltaRa = 0.0f;
								break;
		case GLUT_KEY_UP : 
		case GLUT_KEY_DOWN :	deltaDe = 0.0f;
								break;
		case GLUT_KEY_PAGE_UP :
		case GLUT_KEY_PAGE_DOWN :	deltaOuv = 0;
									break;
	}
}

// ******************  Change flags for the menu  **********************
void GereMenu(int x,int y)
{	int num=x/(32*(float)winW/1024)+1;
	switch (num)
	{	case 1 : FlagConst=!FlagConst; return;
		case 2 : FlagNomConst=!FlagNomConst; return;
		case 3 : FlagNeb=!FlagNeb; return;
		case 4 : FlagLignes=!FlagLignes; return;
		case 5 : FlagLignesAzimuth=!FlagLignesAzimuth; return;
		case 6 : FlagCardinaux=!FlagCardinaux; return;
		case 7 : FlagSol=!FlagSol; return;
		case 8 : FlagHorizon=!FlagHorizon; return;
		case 9 : FlagCiel=!FlagCiel; return;
		case 10 : Flaganim=!Flaganim; return;
		case 11 : Flaganimmin=!Flaganimmin; return;
		case 12 : FlagTexte=!FlagTexte; return;
		case 13 : FlagHelp=!FlagHelp; if (FlagInfo && FlagHelp) FlagInfo=false; return;
	}
}

// ***************  Handle the mouse deplacement  *********************
void BougeSouris(int x, int y)
{	if (y<(float)winH-32*winW/1024 || x>416*(float)winW/1024 || !FlagMenu)
	{	FlagTexteMenu=false;
		return;
	}
	FlagTexteMenu=true;		// If the menu is displayed
	int num=x/(32*(float)winW/1024)+1;			// Calc the current picture under the mouse pointer
	switch (num)
	{	case 1 :  sprintf(stexteMenu,"Drawing of the Constellations"); return;
		case 2 :  sprintf(stexteMenu,"Names of the Constellations"); return;
		case 3 :  sprintf(stexteMenu,"Nebulaes"); return;
		case 4 :  sprintf(stexteMenu,"Equatorial Grid"); return;
		case 5 :  sprintf(stexteMenu,"Azimuthal Grid"); return;
		case 6 :  sprintf(stexteMenu,"Cardinal Points"); return;
		case 7 :  sprintf(stexteMenu,"Ground"); return;
		case 8 :  sprintf(stexteMenu,"Fog"); return;
		case 9 :  sprintf(stexteMenu,"Sky (+ and - to control the brightness)"); return;
		case 10 : sprintf(stexteMenu,"Real Time"); return;
		case 11 : sprintf(stexteMenu,"Accelered Time"); return;
		case 12 : sprintf(stexteMenu,"Text On/Off"); return;
		case 13 : sprintf(stexteMenu,"Help"); return;
	}
return;
}

// *******************  Handle the mouse clic  *************************
void Souris(int button, int state, int x, int y)
{	if (state) return;
	if (button==GLUT_RIGHT_BUTTON) 
	{	FlagSelect=false;
		sprintf(infoObjet,"Selection : -");
		return;
	}
	else
	if (button==GLUT_LEFT_BUTTON)
	{	if (y>winH-(32*(float)winW/1024) && x<(416*(float)winW/1024))
		{	GereMenu(x,y);		// Menu
			return;
		}
		// Else try to select a star
		GLdouble M[16];	
		GLdouble P[16];
		GLdouble objx[1];
		GLdouble objy[1];
		GLdouble objz[1];
		GLint V[4];
		Passe_en_equatorial();
		// Convert x,y screen pos in 3D vector
		glGetDoublev(GL_MODELVIEW_MATRIX,M);
		glGetDoublev(GL_PROJECTION_MATRIX,P);
		glGetIntegerv(GL_VIEWPORT,V);
		if (gluUnProject(x,winH-y,1,M,P,V,objx,objy,objz))
		{	Pointeur[0]=*objx;
			Pointeur[1]=*objy;
			Pointeur[2]=*objz;
			rotPointer=0;
			if (!SystemePlanete.Rechercher(Pointeur) && FlagPlanetes)
			{	FlagSelect=true;
				SystemePlanete.InfoSelect(ObjetSelectionne.XYZ, ObjetSelectionne.RA,ObjetSelectionne.DE, ObjetSelectionne.Nom, ObjetSelectionne.Distance,ObjetSelectionne.Size);
				Pointeur=ObjetSelectionne.XYZ;
				RA_en_hms(ObjetSelectionne.RAh,ObjetSelectionne.RAm,ObjetSelectionne.RAs,ObjetSelectionne.RA);
				ObjetSelectionne.type=2;	//planet type
				ObjetSelectionne.RGB[0]=1.0;
				ObjetSelectionne.RGB[1]=0.3;
				ObjetSelectionne.RGB[2]=0.3;
				ObjetSelectionne.Size=winH*(ObjetSelectionne.Size/5);
			}
			else
			{	if (!messiers.Rechercher(Pointeur) /*&& FlagNeb*/)
				{	FlagSelect=true;
					messiers.InfoSelect(ObjetSelectionne.XYZ,ObjetSelectionne.RA,ObjetSelectionne.DE,ObjetSelectionne.Mag,ObjetSelectionne.Nom,ObjetSelectionne.MessierNum,ObjetSelectionne.NGCNum,ObjetSelectionne.Size);
					Pointeur=ObjetSelectionne.XYZ;
					ObjetSelectionne.Size*=0.8;
					RA_en_hms(ObjetSelectionne.RAh,ObjetSelectionne.RAm,ObjetSelectionne.RAs,ObjetSelectionne.RA);
					ObjetSelectionne.type=1;	//nebulae type
					ObjetSelectionne.RGB[0]=0.4;
					ObjetSelectionne.RGB[1]=0.5;
					ObjetSelectionne.RGB[2]=0.8;
				}
				else
				{	if (!VouteCeleste.Rechercher(Pointeur) && FlagVoute)
					{	FlagSelect=true;
						VouteCeleste.InfoSelect(ObjetSelectionne.XYZ,ObjetSelectionne.RA,ObjetSelectionne.DE,ObjetSelectionne.Mag,ObjetSelectionne.Nom,ObjetSelectionne.HR,ObjetSelectionne.RGB,ObjetSelectionne.CommonName);
						Pointeur=ObjetSelectionne.XYZ;
						RA_en_hms(ObjetSelectionne.RAh,ObjetSelectionne.RAm,ObjetSelectionne.RAs,ObjetSelectionne.RA);
						ObjetSelectionne.type=0;	//star type
						ObjetSelectionne.Size=10;
					}
					else FlagSelect=false;
				}
			}
		}
	}
}

// ****************  Initialisation of glut and openGL  ****************
void init() 
{	glutKeyboardFunc(processNormalKeys);
	glutSpecialFunc(pressKey);
	glutSpecialUpFunc(releaseKey);
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutDisplay);
	glutReshapeFunc(glutResize);
	glutMouseFunc(Souris);
	glutPassiveMotionFunc(BougeSouris);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	glutFullScreen();
	glInit();
	glMatrixMode(GL_MODELVIEW);
}

// ***************************  Main  **********************************
void main (void)
{	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowSize(winW, winH);
	glutCreateWindow(appName);
	AfficheIntro();
	initConfig();

	VouteCeleste.Charger("../data/catalog.fab");	// Load stars
	ConstellCeleste.Charger("../data/constellationsmaj.fab",VouteCeleste);	// Load constellations
	SystemePlanete.Calculer(JDay,LeLieu);			// Compute planet data
	InitMeriParal();								// Precalculation for the grids
	init();
	messiers.Lire("../data/messier.fab");

	glutMainLoop();									// Start drawing
}
