#include "text.h"
#include <string.h>
#include <stdio.h>
#include "GL/glut.h"

extern int texIds[500];
extern int winW;

#define SPACING 1

typedef struct
{	int sizeX;
	int sizeY;
	int spacing;
} fontCharSize;

typedef struct
{	GLuint g_base;
	fontCharSize theSizes[256];
	int fontTexId;
	float Size;
	char fileName[128];
} font;

font font1;
font font2;

// Build Our Font Display List
void build_font(font & theFont)								
{	int posX;
	int posY;
	int sizeX;
	int sizeY;
	int spacing;
	int charNum=0;
	FILE * pFile;

	theFont.g_base = glGenLists(256);							// Creating 256 Display Lists
	glBindTexture(GL_TEXTURE_2D, theFont.fontTexId);			// Select Our Font Texture

	pFile = fopen(theFont.fileName,"rt");
	if (pFile==NULL) printf("ERROR WHILE LOADING %s\n",theFont.fileName);

	while(fscanf(pFile,"%d %d %d %d %d %d\n",&charNum,&posX,&posY,&sizeX,&sizeY,&spacing)==6)
	{	//printf("%d %d %d %d %d %d\n",charNum,posX,posY,sizeX,sizeY,spacing);
		theFont.theSizes[charNum].sizeX=sizeX;
		theFont.theSizes[charNum].sizeY=sizeY;
		theFont.theSizes[charNum].spacing=spacing;
		glNewList(theFont.g_base+charNum,GL_COMPILE);				// Start Building A List
			glTranslated(spacing*theFont.Size,0,0);					// Move To The Left Of The Character
			glBegin(GL_QUADS );
				glTexCoord2f((float)posX/256,(float)(256-posY-sizeY)/256);
				glVertex3f(0,sizeY*theFont.Size,0);					//Bas Gauche
				glTexCoord2f((float)(posX+sizeX)/256,(float)(256-posY-sizeY)/256);	
				glVertex3f(sizeX*theFont.Size,sizeY*theFont.Size,0);				//Bas Droite
				glTexCoord2f((float)(posX+sizeX)/256,(float)(256-posY)/256);
				glVertex3f(sizeX*theFont.Size,0,0);					//Haut Droit
				glTexCoord2f((float)posX/256,(float)(256-posY)/256);
				glVertex3f(0,0,0);						//Haut Gauche
			glEnd ();
			glTranslated((sizeX+SPACING)*theFont.Size,0,0);
		glEndList();									// Done Building The Display List
	}
	fclose(pFile);												
}

// Delete The Font From Memory (NEW)
void kill_font(font theFont)									
{	glDeleteLists(theFont.g_base, 256);							// Delete All 256 Display Lists
}

// Where The Printing Happens (NEW)
void glPrintFont(GLint x, GLint y, char* str, font theFont)	
{
	glBindTexture(GL_TEXTURE_2D, texIds[theFont.fontTexId]);	// Select Our Font Texture
	glPushMatrix();												// Store The Modelview Matrix
	glTranslated(x,y,0);
	glListBase(theFont.g_base);									// Position The Text (0,0 - Top Left)					// Choose The Font Set (0 or 1)
	glCallLists(strlen(str),GL_BYTE,str);						// Write The Text To The Screen
	glPopMatrix();												// Restore The Old Projection Matrix
}

float charSizeOfFont(char * str, font theFont)
{	float s=0;
	for (int i=0;i<(int)strlen(str);i++)
	{	s+=theFont.theSizes[str[i]].sizeX+theFont.theSizes[str[i]].spacing+SPACING;
	}
	return s*theFont.Size;
}

float lineSizeOfFont(char * str, font theFont)
{	float s=0;
	for (int i=0;i<(int)strlen(str);i++)
	{	if (s<theFont.theSizes[str[i]].sizeY) s=theFont.theSizes[str[i]].sizeY;
	}
	return s*theFont.Size;
}

void glPrint(GLint x, GLint y, char* str, int fontNum)
{	if (fontNum<1 || fontNum>2) fontNum=1;
	if (fontNum==1) glPrintFont(x,y,str,font1);
	if (fontNum==2) glPrintFont(x,y,str,font2);
}

void build_fonts(void)
{	font1.fontTexId=30;
	font1.Size=0.75*winW/1024;
	strcpy(font1.fileName,"../data/spacefont.txt");
	build_font(font1);

	font2.fontTexId=30;
	font2.Size=0.6*winW/1024;
	if (winW<700) font2.Size=0.75*winW/1024;
	strcpy(font2.fileName,"../data/spacefont.txt");
	build_font(font2);
}

void kill_fonts(void)
{	kill_font(font1);
	kill_font(font2);
}

float charSizeOf(char * str, int fontNum)
{	if (fontNum<1 || fontNum>2) fontNum=1;
	if (fontNum==1) return charSizeOfFont(str,font1);
	if (fontNum==2) return charSizeOfFont(str,font2);
	return 0;
}

float lineSizeOf(char * str, int fontNum)
{	if (fontNum<1 || fontNum>2) fontNum=1;
	if (fontNum==1) return lineSizeOfFont(str,font1);
	if (fontNum==2) return lineSizeOfFont(str,font2);
	return 0;
}

float charSizeOf(int fontNum)
{	if (fontNum<1 || fontNum>2) fontNum=1;
	if (fontNum==1) return 11*font1.Size;
	if (fontNum==2) return 11*font2.Size;
	return 0;
}

float lineSizeOf(int fontNum)
{	if (fontNum<1 || fontNum>2) fontNum=1;
	if (fontNum==1) return 18*font1.Size;
	if (fontNum==2) return 18*font2.Size;
	return 0;
}
